# Copyright (c) 2009 by David Golden. All rights reserved.
# Licensed under Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License was distributed with this file or you may obtain a 
# copy of the License from http://www.apache.org/licenses/LICENSE-2.0

use strict;
use warnings;

use Test::More ;

my @api = qw(
  capture
  capture_merged
  tee
  tee_merged
);

plan tests => 1 + @api;

require_ok( 'Capture::Tiny' );

can_ok('Capture::Tiny', $_) for @api;

